#!/opt/insight/python/bin/python

import argparse
import glob
import os.path
from argparse import RawTextHelpFormatter

import argcomplete
import sys

from lib.mongo import MongoConnection, delete_jobs_from_mongo, delete_nodes_from_mongo, add_mongo_parser_arguments
from lib.util import query_yes_no


def main(**arguments):
    mongo = MongoConnection(arguments)
    if arguments['object_type'] == 'job':
        mongo.withConnection(delete_jobs_from_mongo, arguments)
    elif arguments['object_type'] == 'node':
        mongo.withConnection(delete_nodes_from_mongo, arguments)


description = """
A script to remove undesired objects from the MongoDB Insight database.

Examples:

./%s job01 job02 job03 --object_type job
./%s node01 node02 node03 --object_type node
echo "node01 node02 node03" | ./%s --object_type node --input -
echo "node01 node02 node03" > file.txt
./%s  --object_type node --input file.txt

Notes:
The script will try to parse /opt/insight/etc/config.groovy to configure the MongoDB connection.
If the script does not find the necessary information use the optional script arguments to override the config file.
If you read objects from standard input or a file using the --input option it also assumes the --force option.
If you want auto-completion of the command line options, make sure to set up python's argcomplete.
To do that run 'eval "$(register-python-argcomplete %s)"' from the same terminal you are running this script.

Warning:
This may affect objects that are related to the object you are deleting. Delete with care because
doing so may result in unintended consequences.


""" % (os.path.basename(__file__), os.path.basename(__file__), os.path.basename(__file__), os.path.basename(__file__),
       os.path.basename(__file__))

if __name__ == '__main__':
    config_file_name = '/opt/insight/etc/config.groovy'
    config = ""
    if os.access(config_file_name, os.R_OK):
        config += " "
        with open(config_file_name, 'r') as f:
            config += f.read().replace('\n', ' ')
    else:
        print "Could not open /opt/insight/etc/config.groovy"

    for config_file_name in glob.glob("/opt/insight/etc/config.d/*"):
        if os.access(config_file_name, os.R_OK):
            config += " "
            with open(config_file_name, 'r') as f:
                config += f.read().replace('\n', ' ')
        else:
            print "Could not open %s" % config_file_name

    parser = argparse.ArgumentParser(description=description, formatter_class=RawTextHelpFormatter)
    parser.add_argument('--input',
                        nargs='?',
                        type=argparse.FileType('r'),
                        help="File that contains space delimited object names. Use the dash(-) to read from "
                             "standard input. If you use this option you will not be asked if you are sure you "
                             "want to delete the objects."
                        )
    parser.add_argument('--object_type',
                        choices=["job", "node"],
                        required=True,
                        help="The type of Insight object that you want to delete from the databases."
                        )
    parser.add_argument('object_names',
                        nargs='*',
                        help="A space delimited list of objects that will be deleted from the Insight databases. "
                             "This is used when the --input option is not used."
                        )
    parser.add_argument('--force',
                        dest='force',
                        action='store_true',
                        help="Do not ask whether or not to delete the objects."
                        )
    add_mongo_parser_arguments(config, parser)
    argcomplete.autocomplete(parser)
    args = parser.parse_args()
    if args.input:
        args.object_names = args.input.read().replace('\n', ' ').split()
        args.input.close()
        del args.input
        args.force = True
    if not args.object_names:
        parser.print_help()
        sys.exit("There were no objects to delete.")

    if args.force:
        main(**vars(args))
    else:
        warning = """
Warning:
Are you sure you want to permanently delete the following %ss %s from these Insight databases?

MongoDB database name: %s
MongoDB username: %s
MongoDB password: ...
MongoDB host: %s
MongoDB port: %s

If not please use the script parameters (use -h for more information).
""" % (
            args.object_type,
            args.object_names,
            args.mongo_database_name,
            args.mongo_username,
            args.mongo_host,
            args.mongo_port
        )
        if query_yes_no(warning):
            main(**vars(args))
