#!/bin/bash

# Estimates the recommended worker.count value according to the disk IO speed.
# This value should be considered as a rough estimate for the target machine. The precise value should be finely tuned 
# with respect to the expected load.

print_help() {
  SELF_CMD="$(basename $0)"

  echo 
  echo "Estimates the recommended worker.count value according to the disk IO speed."
  echo "This value should be considered as a rough estimate for the target machine. The precise value should be finely tuned "
  echo "with respect to the expected load."
  echo
  echo "Command line format: $SELF_CMD <options>"
  echo "Options:"
  echo "    -p, --path"
  echo "           The folder name where the temporary file will be created to test the disk IO speed. Current folder is used by default."
  echo "           NOTE: The folder must be on the same disk where your Postgres stores the data."
  echo "    -i, --iterations"
  echo "           The number of benchmark cycles. The average benchmark result is used. 5 is by default"
  echo "           NOTE: The folder must be on the same disk where your Postgres stores the data."
  echo "    -h, --help"
  echo "           Shows this info screen."
  echo
  echo
}


IOZONE=`which iozone`
WRITES=0
READS=0
ITERS=5
TEST_FOLDER="$(pwd)"

while [[ $# > 0 ]]
do
  key="$1"
  shift
  case $key in
    -i|--iterations)
      ITERS="$1"
      shift
    ;;
    -p|--path)
      TEST_FOLDER="$1"
      shift
    ;;
    *)
      print_help
      exit 0
    ;;
  esac
done

[ -d "$TEST_FOLDER" ] || {
  echo "Cannot access the directory $TEST_FOLDER"
  exit 1
}

[ -x `which python` ] || {
  echo 'python must be installed for this script'
  exit 1
}

[ -x $IOZONE ] || {
  echo 'iozone must be installed'
  exit 1
}

TMP_FILE="$(mktemp --tmpdir="$TEST_FOLDER")"
[ "$?" == "0" ] || {
  echo "Error creating a tempfile in $TEST_FOLDER directory"
  exit 1
}


echo -n "Testing your disk subsystem speed"

for (( i=0; i<$ITERS; i++ )) 
do
  output="$($IOZONE -a -s 8192 -f $TMP_FILE | awk '/^[ ]*[0-9]/ {rndWrite = $8; stride = $11;x=$0} END { print(rndWrite " " stride)}')"
  [ "$?" == "0" ] || {
    echo "FAIL"
    echo "Could not execute disk subsystem benchmark";
    exit 1
  }

  read -a values<<<"$output"
  WRITES="$(bc <<<"$WRITES+${values[0]}")"
  READS="$(bc <<<"$READS+${values[1]}")"
  echo -n "."
done

WRITES="$(bc <<<"$WRITES/$ITERS")"
READS="$(bc <<<"$READS/$ITERS")"

echo "OK"
rm -fv $TMP_FILE >/dev/null 2>&1
read -r -d '' script <<TEXT
import math
import multiprocessing


a=17.2649
b=-3.04053

writes=$WRITES
reads=$READS
meanSpeed=math.sqrt(writes*writes + reads*reads)/1024/1024


K=a/(meanSpeed)+b
K=round(K)
if (K < 1):
  K=1

coreCount=multiprocessing.cpu_count()

print("The recommended worker count is %s (assuming that your machine has %s CPUs)" % (int(K*coreCount), coreCount))
TEXT

#echo "$script"


python <<<"$script"
